#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <timebits.h>
#include <sys/timex.h>
#include <syscall_thunk.h>
#include <errno.h>
#include <asm/unistd.h>

struct x86_timex {
        unsigned int modes;     /* mode selector */
        int  offset;            /* time offset (usec) */
        int  freq;              /* frequency offset (scaled ppm) */
        int  maxerror;          /* maximum error (usec) */
        int  esterror;          /* estimated error (usec) */
        int status;             /* clock command/status */
        int  constant;          /* pll time constant */
        int  precision;         /* clock precision (usec) (read only) */
        int  tolerance;         /* clock frequency tolerance (ppm)
                                 * (read only)
                                 */
        struct timeval time;    /* (read only) */
        int  tick;              /* (modified) usecs between clock ticks */

        int  ppsfreq;           /* pps frequency (scaled ppm) (ro) */
        int  jitter;            /* pps jitter (us) (ro) */
        int shift;              /* interval duration (s) (shift) (ro) */
        int  stabil;            /* pps stability (scaled ppm) (ro) */
        int  jitcnt;            /* jitter limit exceeded (ro) */
        int  calcnt;            /* calibration intervals (ro) */
        int  errcnt;            /* calibration errors (ro) */
        int  stbcnt;            /* stability limit exceeded (ro) */

        int  :32; int  :32; int  :32; int  :32;
        int  :32; int  :32; int  :32; int  :32;
        int  :32; int  :32; int  :32; int  :32;
};


int	do_adjtimex(int a0, int a1, int a2, int a3, int a4, int a5)
{
    struct x86_timex *xtm = ARG_TO_POINTER(a0);
    struct timex      atm;
    int retval;

    /* Convert the supplied timex struct to Alpha format... */
    atm.modes = xtm->modes;
    atm.offset = xtm->offset;
    atm.freq = xtm->freq;
    atm.maxerror = xtm->maxerror;
    atm.esterror = xtm->esterror;
    atm.status = xtm->status;
    atm.constant = xtm->constant;
    atm.precision = xtm->precision;
    atm.tolerance = xtm->tolerance;
    atm.time = xtm->time;    
    atm.tick = xtm->tick;
    atm.jitter = xtm->jitter;
    atm.shift = xtm->shift;
    atm.stabil = xtm->stabil;
    atm.jitcnt = xtm->jitcnt;
    atm.calcnt = xtm->calcnt;
    atm.errcnt = xtm->errcnt;
    atm.stbcnt = xtm->stbcnt;

#ifdef RH_40
    retval = syscall(__NR_adjtimex, &atm);
#else
    retval = adjtimex(&atm);
#endif

    if(retval == -1) {
	return(-alpha_to_intel_errno(errno));
    }
    else {
	xtm->modes = atm.modes;
	xtm->offset = atm.offset;
	xtm->freq = atm.freq;
	xtm->maxerror = atm.maxerror;
	xtm->esterror = atm.esterror;
	xtm->status = atm.status;
	xtm->constant = atm.constant;
	xtm->precision = atm.precision;
	xtm->tolerance = atm.tolerance;
	xtm->time = atm.time;    
	xtm->tick = atm.tick;
	xtm->jitter = atm.jitter;
	xtm->shift = atm.shift;
	xtm->stabil = atm.stabil;
	xtm->jitcnt = atm.jitcnt;
	xtm->calcnt = atm.calcnt;
	xtm->errcnt = atm.errcnt;
	xtm->stbcnt = atm.stbcnt;

	return(retval);
    }
}

