/*
 * em86 Linux Runtime Emulator
 *
 * File     : ioctl_thunk.h
 * Author   : Adam Stachelek
 * Overview :
 *
 * This is the header file for the various ioctl processing routines.
 */

#ifndef _EM86_IOCTL_THUNK_H
#define _EM86_IOCTL_THUNK_H 1

#include <syscall_thunk.h>

/* Intel Linux values for the support ioctls */

#define X86_TCGETS          0x5401
#define X86_TCSETS          0x5402
#define X86_TCSETSW         0x5403
#define X86_TCSETSF         0x5404
#define X86_TCGETA          0x5405
#define X86_TCSETA          0x5406
#define X86_TCSETAW         0x5407
#define X86_TCSETAF         0x5408
#define X86_TCSBRK          0x5409
#define X86_TCXONC          0x540A
#define X86_TCFLSH          0x540B
#define X86_TIOCEXCL        0x540C
#define X86_TIOCNXCL        0x540D
#define X86_TIOCSCTTY       0x540E
#define X86_TIOCGPGRP       0x540F
#define X86_TIOCSPGRP       0x5410
#define X86_TIOCOUTQ        0x5411
#define X86_TIOCSTI         0x5412
#define X86_TIOCGWINSZ      0x5413
#define X86_TIOCSWINSZ      0x5414
#define X86_TIOCMGET        0x5415
#define X86_TIOCMBIS        0x5416
#define X86_TIOCMBIC        0x5417
#define X86_TIOCMSET        0x5418
#define X86_TIOCGSOFTCAR    0x5419
#define X86_TIOCSSOFTCAR    0x541A
#define X86_FIONREAD        0x541B
#define X86_TIOCINQ         X86_FIONREAD
#define X86_TIOCLINUX       0x541C
#define X86_TIOCCONS        0x541D
#define X86_TIOCGSERIAL     0x541E
#define X86_TIOCSSERIAL     0x541F
#define X86_TIOCPKT         0x5420
#define X86_FIONBIO         0x5421
#define X86_TIOCNOTTY       0x5422
#define X86_TIOCSETD        0x5423
#define X86_TIOCGETD        0x5424
#define X86_TCSBRKP         0x5425  /* Needed for POSIX tcsendbreak() */
#define X86_TIOCTTYGSTRUCT  0x5426  /* For debugging only */
#define X86_FIONCLEX        0x5450  /* these numbers need to be adjusted. */
#define X86_FIOCLEX         0x5451
#define X86_FIOASYNC        0x5452
#define X86_TIOCSERCONFIG   0x5453
#define X86_TIOCSERGWILD    0x5454
#define X86_TIOCSERSWILD    0x5455
#define X86_TIOCGLCKTRMIOS  0x5456
#define X86_TIOCSLCKTRMIOS  0x5457
#define X86_TIOCSERGSTRUCT  0x5458  /* For debugging only */
#define X86_TIOCSERGETLSR   0x5459  /* Get line status register */
#define X86_TIOCSERGETMULTI 0x545A  /* Get multiport config  */
#define X86_TIOCSERSETMULTI 0x545B  /* Set multiport config */
#define X86_TIOCMIWAIT      0x545C  /* wait for a change on serial input line(s) */
#define X86_TIOCGICOUNT     0x545D  /* read serial port inline interrupt counts */

/* Networking ioctls */
#define X86_SIOCADDRT       0x890B          /* add routing table entry */
#define X86_SIOCDELRT       0x890C          /* delete routing table entry */
#define X86_SIOCGIFNAME     0x8910          /* get iface name               */
#define X86_SIOCSIFLINK     0x8911          /* set iface channel            */
#define X86_SIOCGIFCONF     0x8912          /* get iface list               */
#define X86_SIOCGIFFLAGS    0x8913          /* get flags                    */
#define X86_SIOCSIFFLAGS    0x8914          /* set flags                    */
#define X86_SIOCGIFADDR     0x8915          /* get PA address               */
#define X86_SIOCSIFADDR     0x8916          /* set PA address               */
#define X86_SIOCGIFDSTADDR  0x8917          /* get remote PA address        */
#define X86_SIOCSIFDSTADDR  0x8918          /* set remote PA address        */
#define X86_SIOCGIFBRDADDR  0x8919          /* get broadcast PA address     */
#define X86_SIOCSIFBRDADDR  0x891a          /* set broadcast PA address     */
#define X86_SIOCGIFNETMASK  0x891b          /* get network PA mask          */
#define X86_SIOCSIFNETMASK  0x891c          /* set network PA mask          */
#define X86_SIOCGIFMETRIC   0x891d          /* get metric                   */
#define X86_SIOCSIFMETRIC   0x891e          /* set metric                   */
#define X86_SIOCGIFMEM      0x891f          /* get memory address (BSD)     */
#define X86_SIOCSIFMEM      0x8920          /* set memory address (BSD)     */
#define X86_SIOCGIFMTU      0x8921          /* get MTU size                 */
#define X86_SIOCSIFMTU      0x8922          /* set MTU size                 */
#define X86_SIOCSIFHWADDR   0x8924          /* set hardware address (NI)    */
#define X86_SIOCGIFENCAP    0x8925          /* get/set slip encapsulation   */
#define X86_SIOCSIFENCAP    0x8926
#define X86_SIOCGIFHWADDR   0x8927          /* Get hardware address         */
#define X86_SIOCGIFSLAVE    0x8929          /* Driver slaving support       */
#define X86_SIOCSIFSLAVE    0x8930
#define X86_SIOCADDMULTI    0x8931          /* Multicast address lists      */
#define X86_SIOCDELMULTI    0x8932

/* Bridging control calls */
#define X86_SIOCGIFBR       0x8940          /* Bridging support             */
#define X86_SIOCSIFBR       0x8941          /* Set bridging options         */


/* ARP cache control calls. */
#define X86_OLD_SIOCDARP    0x8950          /* old delete ARP table entry   */
#define X86_OLD_SIOCGARP    0x8951          /* old get ARP table entry      */
#define X86_OLD_SIOCSARP    0x8952          /* old set ARP table entry      */
#define X86_SIOCDARP        0x8953          /* delete ARP table entry       */
#define X86_SIOCGARP        0x8954          /* get ARP table entry          */
#define X86_SIOCSARP        0x8955          /* set ARP table entry          */

/* RARP cache control calls. */
#define X86_SIOCDRARP       0x8960          /* delete RARP table entry      */
#define X86_SIOCGRARP       0x8961          /* get RARP table entry         */
#define X86_SIOCSRARP       0x8962          /* set RARP table entry         */

/* Driver configuration calls */
#define X86_SIOCGIFMAP      0x8970          /* Get device parameters        */
#define X86_SIOCSIFMAP      0x8971          /* Set device parameters        */

/* DLCI configuration calls */
#define X86_SIOCADDDLCI     0x8980          /* Create new DLCI device       */
#define X86_SIOCDELDLCI     0x8981          /* Delete DLCI device           */

/* General CDROM control calls */
#define X86_CDROMPAUSE		0x5301
#define X86_CDROMRESUME		0x5302
#define X86_CDROMPLAYMSF	0x5303	/* (struct cdrom_msf) */
#define X86_CDROMPLAYTRKIND	0x5304	/* (struct cdrom_ti) */

#define X86_CDROMREADTOCHDR	0x5305	/* (struct cdrom_tochdr) */
#define X86_CDROMREADTOCENTRY	0x5306	/* (struct cdrom_tocentry) */

#define X86_CDROMSTOP		0x5307	/* stop the drive motor	*/
#define X86_CDROMSTART		0x5308	/* turn the motor on */

#define X86_CDROMEJECT		0x5309	/* eject CD-ROM media */

#define X86_CDROMVOLCTRL	0x530a	/* (struct cdrom_volctrl) */

#define X86_CDROMSUBCHNL	0x530b	/* (struct cdrom_subchnl) */

#define X86_CDROMREADMODE2	0x530c	/* (struct cdrom_read) */
                                        /* read type-2 data */

#define X86_CDROMREADMODE1	0x530d	/* (struct cdrom_read) */
                                        /* read type-1 data */

#define X86_CDROMREADAUDIO	0x530e	/* (struct cdrom_read_audio) */

/*
 * enable (1) / disable (0) auto-ejecting
 */
#define X86_CDROMEJECT_SW	0x530f	/* arg: 0 or 1 */
 
/*
 * obtain the start-of-last-session address of multi session disks
 */
#define X86_CDROMMULTISESSION	0x5310	/* (struct cdrom_multisession) */

/*
 * obtain the "universal product code" number
 * (only some data disks have it coded)
 */
#define X86_CDROM_GET_UPC	0x5311	/* 8 bytes returned */

#define X86_CDROMRESET		0x5312	/* hard-reset the drive */
#define X86_CDROMVOLREAD	0x5313	/* let the drive tell its volume setting */
					/* (struct cdrom_volctrl) */

/* 
 * these ioctls are used in aztcd.c and optcd.c
 */
#define X86_CDROMREADRAW	0x5314	/* read data in raw mode */
#define X86_CDROMREADCOOKED	0x5315	/* read data in cooked mode */
#define X86_CDROMSEEK		0x5316  /* seek msf address */
  
/*
 * for playing audio in logical block addressing mode
 */
#define X86_CDROMPLAYBLK	0x5317	/* (struct cdrom_blk) */

/* 
 * these ioctls are used in optcd.c
 */
#define X86_CDROMREADALL	0x5318	/* read all 2646 bytes */
#define X86_CDROMCLOSETRAY	0x5319	/* pendant of CDROMEJECT */

#define X86_CDROM_SET_OPTIONS       0x5320
#define X86_CDROM_CLEAR_OPTIONS     0x5321
#define X86_CDROM_SELECT_SPEED      0x5322  /* head-speed */
#define X86_CDROM_SELECT_DISC       0x5323  /* for juke-boxes */
#define X86_CDROM_MEDIA_CHANGED     0x5325
#define X86_CDROM_DRIVE_STATUS      0x5326  /* tray position, etc. */
#define X86_CDROM_DISC_STATUS       0x5327  /* disc type etc. */

/* Rename an old ioctl */
#define X86_CDROM_GET_MCN   X86_CDROM_GET_UPC   /* medium catalog number */

/* For controlling sbpcd CDROM's */
#define X86_CDROMAUDIOBUFSIZ        0x5382 /* set the audio buffer size */

/* Sound ioctls */
#define X86_SNDCTL_COPR_HALT           0xc0144307
#define X86_SNDCTL_COPR_LOAD           0xcfb04301
#define X86_SNDCTL_COPR_RCODE          0xc0144303
#define X86_SNDCTL_COPR_RCVMSG         0x8fa44309
#define X86_SNDCTL_COPR_RDATA          0xc0144302
#define X86_SNDCTL_COPR_RESET          0x00004300
#define X86_SNDCTL_COPR_RUN            0xc0144306
#define X86_SNDCTL_COPR_SENDMSG        0xcfa44308
#define X86_SNDCTL_COPR_WCODE          0x40144305
#define X86_SNDCTL_COPR_WDATA          0x40144304
#define X86_SNDCTL_DSP_CHANNELS        0xc0045006
#define X86_SNDCTL_DSP_GETBLKSIZE      0xc0045004
#define X86_SNDCTL_DSP_GETCAPS         0x8004500f
#define X86_SNDCTL_DSP_GETFMTS         0x8004500b
#define X86_SNDCTL_DSP_GETIPTR         0x800c5011
#define X86_SNDCTL_DSP_GETISPACE       0x8010500d
#define X86_SNDCTL_DSP_GETOPTR         0x800c5012
#define X86_SNDCTL_DSP_GETOSPACE       0x8010500c
#define X86_SNDCTL_DSP_GETTRIGGER      0x80045010
#define X86_SNDCTL_DSP_MAPINBUF        0x80085013
#define X86_SNDCTL_DSP_MAPOUTBUF       0x80085014
#define X86_SNDCTL_DSP_NONBLOCK        0x0000500e
#define X86_SNDCTL_DSP_POST            0x00005008
#define X86_SNDCTL_DSP_RESET           0x00005000
#define X86_SNDCTL_DSP_SAMPLESIZE      0xc0045005
#define X86_SNDCTL_DSP_SETDUPLEX       0x00005016
#define X86_SNDCTL_DSP_SETFMT          0xc0045005
#define X86_SNDCTL_DSP_SETFRAGMENT     0xc004500a
#define X86_SNDCTL_DSP_SETSYNCRO       0x00005015
#define X86_SNDCTL_DSP_SETTRIGGER      0x40045010
#define X86_SNDCTL_DSP_SPEED           0xc0045002
#define X86_SNDCTL_DSP_STEREO          0xc0045003
#define X86_SNDCTL_DSP_SUBDIVIDE       0xc0045009
#define X86_SNDCTL_DSP_SYNC            0x00005001
#define X86_SNDCTL_FM_4OP_ENABLE       0x4004510f
#define X86_SNDCTL_FM_LOAD_INSTR       0x40285107
#define X86_SNDCTL_MIDI_INFO           0xc074510c
#define X86_SNDCTL_MIDI_MPUCMD         0xc0216d02
#define X86_SNDCTL_MIDI_MPUMODE        0xc0046d01
#define X86_SNDCTL_MIDI_PRETIME        0xc0046d00
#define X86_SNDCTL_PMGR_ACCESS         0xcfb85110
#define X86_SNDCTL_PMGR_IFACE          0xcfb85001
#define X86_SNDCTL_SEQ_CTRLRATE        0xc0045103
#define X86_SNDCTL_SEQ_GETINCOUNT      0x80045105
#define X86_SNDCTL_SEQ_GETOUTCOUNT     0x80045104
#define X86_SNDCTL_SEQ_NRMIDIS         0x8004510b
#define X86_SNDCTL_SEQ_NRSYNTHS        0x8004510a
#define X86_SNDCTL_SEQ_OUTOFBAND       0x40085112
#define X86_SNDCTL_SEQ_PANIC           0x00005111
#define X86_SNDCTL_SEQ_PERCMODE        0x40045106
#define X86_SNDCTL_SEQ_RESET           0x00005100
#define X86_SNDCTL_SEQ_RESETSAMPLES    0x40045109
#define X86_SNDCTL_SEQ_SYNC            0x00005101
#define X86_SNDCTL_SEQ_TESTMIDI        0x40045108
#define X86_SNDCTL_SEQ_THRESHOLD       0x4004510d
#define X86_SNDCTL_SEQ_TRESHOLD        0x4004510d
#define X86_SNDCTL_SYNTH_INFO          0xc08c5102
#define X86_SNDCTL_SYNTH_MEMAVL        0xc004510e
#define X86_SNDCTL_TMR_CONTINUE        0x00005404
#define X86_SNDCTL_TMR_METRONOME       0x40045407
#define X86_SNDCTL_TMR_SELECT          0x40045408
#define X86_SNDCTL_TMR_SOURCE          0xc0045406
#define X86_SNDCTL_TMR_START           0x00005402
#define X86_SNDCTL_TMR_STOP            0x00005403
#define X86_SNDCTL_TMR_TEMPO           0xc0045405
#define X86_SNDCTL_TMR_TIMEBASE        0xc0045401
#define X86_SOUND_PCM_WRITE_FILTER     0xc0045007
#define X86_SOUND_PCM_READ_RATE        0x80045002
#define X86_SOUND_PCM_READ_CHANNELS    0x80045006
#define X86_SOUND_PCM_READ_BITS        0x80045005
#define X86_SOUND_PCM_READ_FILTER      0x80045007
#define X86_SOUND_MIXER_INFO           0x80304d65
#define X86_SOUND_MIXER_ACCESS         0xc0804d66
#define X86_SOUND_MIXER_PRIVATE1       0xc0044d6f
#define X86_SOUND_MIXER_PRIVATE2       0xc0044d70
#define X86_SOUND_MIXER_PRIVATE3       0xc0044d71
#define X86_SOUND_MIXER_PRIVATE4       0xc0044d72
#define X86_SOUND_MIXER_PRIVATE5       0xc0044d73
#define X86_SOUND_MIXER_READ_VOLUME    0x80044d00
#define X86_SOUND_MIXER_READ_BASS      0x80044d01
#define X86_SOUND_MIXER_READ_TREBLE    0x80044d02
#define X86_SOUND_MIXER_READ_SYNTH     0x80044d03
#define X86_SOUND_MIXER_READ_PCM       0x80044d04
#define X86_SOUND_MIXER_READ_SPEAKER   0x80044d05
#define X86_SOUND_MIXER_READ_LINE      0x80044d06
#define X86_SOUND_MIXER_READ_MIC       0x80044d07
#define X86_SOUND_MIXER_READ_CD        0x80044d08
#define X86_SOUND_MIXER_READ_IMIX      0x80044d09
#define X86_SOUND_MIXER_READ_ALTPCM    0x80044d0a
#define X86_SOUND_MIXER_READ_RECLEV    0x80044d0b
#define X86_SOUND_MIXER_READ_IGAIN     0x80044d0c
#define X86_SOUND_MIXER_READ_OGAIN     0x80044d0d
#define X86_SOUND_MIXER_READ_LINE1     0x80044d0e
#define X86_SOUND_MIXER_READ_LINE2     0x80044d0f
#define X86_SOUND_MIXER_READ_LINE3     0x80044d10
#define X86_SOUND_MIXER_READ_MUTE      0x80044d1f
#define X86_SOUND_MIXER_READ_ENHANCE   0x80044d1f
#define X86_SOUND_MIXER_READ_LOUD      0x80044d1f
#define X86_SOUND_MIXER_READ_RECSRC    0x80044dff
#define X86_SOUND_MIXER_READ_DEVMASK   0x80044dfe
#define X86_SOUND_MIXER_READ_RECMASK   0x80044dfd
#define X86_SOUND_MIXER_READ_STEREODEVS  0x80044dfb
#define X86_SOUND_MIXER_READ_CAPS      0x80044dfc
#define X86_SOUND_MIXER_WRITE_VOLUME   0xc0044d00
#define X86_SOUND_MIXER_WRITE_BASS     0xc0044d01
#define X86_SOUND_MIXER_WRITE_TREBLE   0xc0044d02
#define X86_SOUND_MIXER_WRITE_SYNTH    0xc0044d03
#define X86_SOUND_MIXER_WRITE_PCM      0xc0044d04
#define X86_SOUND_MIXER_WRITE_SPEAKER  0xc0044d05
#define X86_SOUND_MIXER_WRITE_LINE     0xc0044d06
#define X86_SOUND_MIXER_WRITE_MIC      0xc0044d07
#define X86_SOUND_MIXER_WRITE_CD       0xc0044d08
#define X86_SOUND_MIXER_WRITE_IMIX     0xc0044d09
#define X86_SOUND_MIXER_WRITE_ALTPCM   0xc0044d0a
#define X86_SOUND_MIXER_WRITE_RECLEV   0xc0044d0b
#define X86_SOUND_MIXER_WRITE_IGAIN    0xc0044d0c
#define X86_SOUND_MIXER_WRITE_OGAIN    0xc0044d0d
#define X86_SOUND_MIXER_WRITE_LINE1    0xc0044d0e
#define X86_SOUND_MIXER_WRITE_LINE2    0xc0044d0f
#define X86_SOUND_MIXER_WRITE_LINE3    0xc0044d10
#define X86_SOUND_MIXER_WRITE_MUTE     0xc0044d1f
#define X86_SOUND_MIXER_WRITE_ENHANCE  0xc0044d1f
#define X86_SOUND_MIXER_WRITE_LOUD     0xc0044d1f
#define X86_SOUND_MIXER_WRITE_RECSRC   0xc0044dff




/* HD commands */

/* hd/ide ctl's that pass (arg) ptrs to user space are numbered 0x030n/0x031n */
#define X86_HDIO_GETGEO            0x0301  /* get device geometry */
#define X86_HDIO_GET_UNMASKINTR    0x0302  /* get current unmask setting */
#define X86_HDIO_GET_MULTCOUNT     0x0304  /* get current IDE blockmode setting */
#define X86_HDIO_GET_IDENTITY      0x0307  /* get IDE identification info */
#define X86_HDIO_GET_KEEPSETTINGS  0x0308  /* get keep-settings-on-reset flag */
#define X86_HDIO_GET_32BIT         0x0309  /* get current io_32bit setting */
#define X86_HDIO_GET_NOWERR        0x030a  /* get ignore-write-error flag */
#define X86_HDIO_GET_DMA           0x030b  /* get use-dma flag */
#define X86_HDIO_DRIVE_CMD         0x031f  /* execute a special drive command */

/* hd/ide ctl's that pass (arg) non-ptr values are numbered 0x032n/0x033n */
#define X86_HDIO_SET_MULTCOUNT     0x0321  /* change IDE blockmode */
#define X86_HDIO_SET_UNMASKINTR    0x0322  /* permit other irqs during I/O */
#define X86_HDIO_SET_KEEPSETTINGS  0x0323  /* keep ioctl settings on reset */
#define X86_HDIO_SET_32BIT         0x0324  /* change io_32bit flags */
#define X86_HDIO_SET_NOWERR        0x0325  /* change ignore-write-error flag */
#define X86_HDIO_SET_DMA           0x0326  /* change use-dma flag */
#define X86_HDIO_SET_PIO_MODE      0x0327  /* reconfig interface to new speed */

/* From <linux/fs.h> */

#define X86_BLKROSET  0x125d       /* set device read-only (0 = read-write) */
#define X86_BLKROGET  0x125e       /* get read-only status (0 = read_write) */
#define X86_BLKRRPART 0x125f       /* re-read partition table */
#define X86_BLKGETSIZE 0x1260       /* return device size */
#define X86_BLKFLSBUF 0x1261       /* flush buffer cache */
#define X86_BLKRASET  0x1262       /* Set read ahead for block device */
#define X86_BLKRAGET  0x1263       /* get current read ahead setting */
#define X86_FIBMAP    0x0001       /* bmap access */
#define X86_FIGETBSZ  0x0002       /* get the block size used for bmap */


/* termios bit definitions */

/* c_iflag bits */
#define X86_IGNBRK  0000001
#define X86_BRKINT  0000002
#define X86_IGNPAR  0000004
#define X86_PARMRK  0000010
#define X86_INPCK   0000020
#define X86_ISTRIP  0000040
#define X86_INLCR   0000100
#define X86_IGNCR   0000200
#define X86_ICRNL   0000400
#define X86_IUCLC   0001000
#define X86_IXON    0002000
#define X86_IXANY   0004000
#define X86_IXOFF   0010000
#define X86_IMAXBEL 0020000

/* c_oflag bits */
#define X86_OPOST   0000001
#define X86_OLCUC   0000002
#define X86_ONLCR   0000004
#define X86_OCRNL   0000010
#define X86_ONOCR   0000020
#define X86_ONLRET  0000040
#define X86_OFILL   0000100
#define X86_OFDEL   0000200
#define X86_NLDLY   0000400
#define   X86_NL0   0000000
#define   X86_NL1   0000400
#define X86_CRDLY   0003000
#define   X86_CR0   0000000
#define   X86_CR1   0001000
#define   X86_CR2   0002000
#define   X86_CR3   0003000
#define X86_TABDLY  0014000
#define   X86_TAB0  0000000
#define   X86_TAB1  0004000
#define   X86_TAB2  0010000
#define   X86_TAB3  0014000
#define   X86_XTABS 0014000
#define X86_BSDLY   0020000
#define   X86_BS0   0000000
#define   X86_BS1   0020000
#define X86_VTDLY   0040000
#define   X86_VT0   0000000
#define   X86_VT1   0040000
#define X86_FFDLY   0100000
#define   X86_FF0   0000000
#define   X86_FF1   0100000

/* c_cflag bit meaning */
#define X86_CBAUD   0010017
#define  X86_B0     0000000         /* hang up */
#define  X86_B50    0000001
#define  X86_B75    0000002
#define  X86_B110   0000003
#define  X86_B134   0000004
#define  X86_B150   0000005
#define  X86_B200   0000006
#define  X86_B300   0000007
#define  X86_B600   0000010
#define  X86_B1200  0000011
#define  X86_B1800  0000012
#define  X86_B2400  0000013
#define  X86_B4800  0000014
#define  X86_B9600  0000015
#define  X86_B19200 0000016
#define  X86_B38400 0000017
#define X86_EXTA B19200
#define X86_EXTB B38400
#define X86_CSIZE   0000060
#define   X86_CS5   0000000
#define   X86_CS6   0000020
#define   X86_CS7   0000040
#define   X86_CS8   0000060
#define X86_CSTOPB  0000100
#define X86_CREAD   0000200
#define X86_PARENB  0000400
#define X86_PARODD  0001000
#define X86_HUPCL   0002000
#define X86_CLOCAL  0004000
#define X86_CBAUDEX 0010000
#define  X86_B57600  0010001
#define  X86_B115200 0010002
#define  X86_B230400 0010003
#define  X86_B460800 0010004
#define X86_CIBAUD    002003600000  /* input baud rate (not used) */
#define X86_CRTSCTS   020000000000          /* flow control */

/* c_lflag bits */
#define X86_ISIG    0000001
#define X86_ICANON  0000002
#define X86_XCASE   0000004
#define X86_ECHO    0000010
#define X86_ECHOE   0000020
#define X86_ECHOK   0000040
#define X86_ECHONL  0000100
#define X86_NOFLSH  0000200
#define X86_TOSTOP  0000400
#define X86_ECHOCTL 0001000
#define X86_ECHOPRT 0002000
#define X86_ECHOKE  0004000
#define X86_FLUSHO  0010000
#define X86_PENDIN  0040000
#define X86_IEXTEN  0100000

/* c_cc character offsets */
#define X86_VINTR	0
#define X86_VQUIT	1
#define X86_VERASE	2
#define X86_VKILL	3
#define X86_VEOF	4
#define X86_VTIME	5
#define X86_VMIN	6
#define X86_VSWTC	7
#define X86_VSTART	8
#define X86_VSTOP	9
#define X86_VSUSP	10
#define X86_VEOL	11
#define X86_VREPRINT	12
#define X86_VDISCARD	13
#define X86_VWERASE	14
#define X86_VLNEXT	15
#define X86_VEOL2	16

/* Miscellaneous definitions */

#define MAX_STRUCT_SIZE        4096
#define X86_TERMIOS_SIZE       36
#define ARRAY_TYPE_SHIFT       16
#define ARRAY_SIZE_MASK       0xffff
#define GET_ARRAY_TYPE(t)      (t >> ARRAY_TYPE_SHIFT)
#define GET_ARRAY_SIZE(s)      (s & ARRAY_SIZE_MASK)
#define MK_ARRAY_INFO(t,s)     ((t << ARRAY_TYPE_SHIFT)|(s & ARRAY_SIZE_MASK))

/* Macros for thunking 32-bit to 64-bit quantities */

#define intel_to_alpha_ptr(ptr) ((unsigned long) ptr & 0xffffffff)
#define intel_to_alpha_long(val) intel_to_alpha_ptr(val)

/* argtype: specifies listing of type identifiers
 * NOTE: the entries after REGSTRUCT must not have their order changed,
 * as they correspond to entries in struct_table */

typedef enum {
  TYPE_NULL,
  TYPE_CHAR,
  TYPE_SHORT,
  TYPE_INT,
  TYPE_LONG,
  TYPE_ULONG,
  TYPE_PTR,
  TYPE_ARRAY,
  SPECSTRUCT,      /* This marker shows beginning of special struct region */
  TYPE_TERMIOS,
  REGSTRUCT,       /* This marker shows all above are regular structs */
  TYPE_WINSIZE,
  TYPE_TQ_STRUCT,
  TYPE_ASYNC_STRUCT,
  TYPE_ASYNC_ICOUNT,
  TYPE_TTY_STRUCT,
  TYPE_TTY_DRIVER,
  TYPE_TTY_LDISC,
  TYPE_TTY_FLIP_BUFFER,
  TYPE_SERIAL_MULTIPORT_STRUCT,
  TYPE_SERIAL_ICOUNTER_STRUCT,
  TYPE_SOCKADDR,
  TYPE_RTENTRY,
  TYPE_IFMAP,
  TYPE_SOCKADDR_IFREQ,
  TYPE_SHORT_IFREQ,
  TYPE_INT_IFREQ,
  TYPE_IFMAP_IFREQ,
  TYPE_CHAR_IFREQ,
  TYPE_PTR_IFREQ,
  TYPE_IFCONF,
  TYPE_ARPREQ,
  TYPE_ARPREQ_OLD,
  TYPE_BR_STAT,
  TYPE_BRIDGE_DATA,
  TYPE_PORT_DATA,
  TYPE_BRIDGE_ID_T,
  TYPE_CDROM_READ_AUDIO,
  TYPE_HD_GEOMETRY
} argtype;

/* ioctl_arguse: defines the device driver's usage of the argument: read-only,
 * return value back, or both */

typedef struct {
  int x86_cmd;
  int alpha_cmd;
  char *name;
  int (*routine)();
  /* keep this next field until some standards are imposed on ioctl #'s */
  int arguse;
  argtype *list;
} ioctl_descriptor;

/* Below are all the tables needed by ioctl.c from ioctl_list.c */

extern bitmask_transtbl iflag_tbl [];
extern bitmask_transtbl oflag_tbl [];
extern bitmask_transtbl cflag_tbl [];
extern bitmask_transtbl lflag_tbl [];

extern ioctl_descriptor ioctl_table [];

extern argtype async_struct_list [];
extern argtype tq_struct_list [];
extern argtype async_icount_list [];
extern argtype tty_struct_list [];
extern argtype tty_flip_buffer_list [];
extern argtype tty_driver_list [];
extern argtype tty_ldisc_list [];
extern argtype winsize_list [];
extern argtype serial_multiport_struct_list [];
extern argtype serial_icounter_struct_list [];
extern argtype sockaddr_list [];
extern argtype rtentry_list [];
extern argtype ifmap_list [];
extern argtype sockaddr_ifreq_list [];
extern argtype short_ifreq_list [];
extern argtype int_ifreq_list [];
extern argtype ifmap_ifreq_list [];
extern argtype char_ifreq_list [];
extern argtype ptr_ifreq_list [];
extern argtype ifconf_list [];
extern argtype arpreq_list [];
extern argtype arpreq_old_list [];
extern argtype br_stat_list [];
extern argtype Bridge_data_list [];
extern argtype Port_data_list [];
extern argtype bridge_id_t_list [];
extern argtype cdrom_read_audio [];
extern argtype hd_geometry_list [];

extern argtype *struct_table [];

/* Forward prototypes of ioctl.c routines, these could probably be put
 * in ioctl.c */

int do_ptr_ioctl (int fd, ioctl_descriptor desc, int arg);
int do_struct_ioctl (int fd, ioctl_descriptor desc, int arg);
int do_termios_ioctl (int fd, ioctl_descriptor desc, int arg);
int do_int_ioctl (int fd, ioctl_descriptor desc, int arg);
int do_longptr_ioctl (int fd, ioctl_descriptor desc, int arg);
int do_void_ioctl (int fd, ioctl_descriptor desc, int arg);

#endif







