#include <sys/types.h>
#include <sys/ipc_buf.h>
#include <sys/msq_buf.h>
#include <sys/sem_buf.h>
#include <sys/shm_buf.h>
#include <syscall_thunk.h>
#include <errno.h>

/*
 * Syscall thunk for the x86 ipc() syscall.  Pieces stolen from the kernel
 * sources (arch/i386/kernel/sys_i386.c)
 *
 * This is really horribly ugly.
 */

#define X86_SEMOP           1
#define X86_SEMGET          2
#define X86_SEMCTL          3 
#define X86_MSGSND          11 
#define X86_MSGRCV          12
#define X86_MSGGET          13
#define X86_MSGCTL          14
#define X86_SHMAT           21
#define X86_SHMDT           22
#define X86_SHMGET          23
#define X86_SHMCTL          24

struct x86_msgbuf {
	int mtype;
	char mtext[1];
};

struct x86_ipc_kludge {
	unsigned int	msgp;	/* Really (struct msgbuf *) */
	int msgtyp;
};	

struct alpha_msgbuf {
	long	mtype;
	char	mtext[4096];
};

struct x86_ipc_perm {
	int	key;
	unsigned short	uid;
	unsigned short	gid;
	unsigned short	cuid;
	unsigned short	cgid;
	unsigned short	mode;
	unsigned short	seq;
};

struct x86_msqid_ds {
	struct x86_ipc_perm	msg_perm;
	unsigned int		msg_first;	/* really struct x86_msg* */
	unsigned int		msg_last;	/* really struct x86_msg* */
	unsigned int		msg_stime;	/* really x86_time_t */
	unsigned int		msg_rtime;	/* really x86_time_t */
	unsigned int		msg_ctime;	/* really x86_time_t */
	unsigned int		wwait;		/* really struct wait_queue* */
	unsigned int		rwait;		/* really struct wait_queue* */
	unsigned short		msg_cbytes;
	unsigned short		msg_qnum;
	unsigned short		msg_qbytes;
	unsigned short		msg_lspid;
	unsigned short		msg_lrpid;
};

struct x86_shmid_ds {
	struct x86_ipc_perm	shm_perm;
	int			shm_segsz;
	unsigned int		shm_atime;	/* really x86_time_t */
	unsigned int		shm_dtime;	/* really x86_time_t */
	unsigned int		shm_ctime;	/* really x86_time_t */
	unsigned short		shm_cpid;
	unsigned short		shm_lpid;
	short			shm_nattch;
	unsigned short		shm_npages;
	unsigned long		*shm_pages;
	void 			*attaches;	/* really struct shm_desc * */
};

#define X86_IPC_RMID	0
#define X86_IPC_SET	1
#define X86_IPC_STAT	2

union x86_semun {
    int val;
    unsigned int buf;	/* really struct semid_ds * */
    unsigned int array; /* really unsigned short * */
    unsigned int __buf;	/* really struct seminfo * */
    unsigned int __pad;	/* really void* */
};

int do_ipc (unsigned int call, int first, int second, int third, unsigned int xptr, int fifth)
{
    int version = call >> 16;
    void *	ptr = ARG_TO_POINTER(xptr);

    call &= 0xffff;

    switch(call) {
	case X86_SEMOP:		return do_semop(version, first, second, third, ptr, fifth);
	case X86_SEMGET:	return do_semget(version, first, second, third, ptr, fifth);
	case X86_SEMCTL:	return do_semctl(version, first, second, third, ptr, fifth);
	case X86_MSGSND:	return do_msgsnd(version, first, second, third, ptr, fifth);
	case X86_MSGRCV:	return do_msgrcv(version, first, second, third, ptr, fifth);
	case X86_MSGGET:	return do_msgget(version, first, second, third, ptr, fifth);
	case X86_MSGCTL:	return do_msgctl(version, first, second, third, ptr, fifth);
	case X86_SHMAT:		return do_shmat(version, first, second, third, ptr, fifth);
	case X86_SHMDT:		return do_shmdt(version, first, second, third, ptr, fifth);
	case X86_SHMGET:	return do_shmget(version, first, second, third, ptr, fifth);
	case X86_SHMCTL:	return do_shmctl(version, first, second, third, ptr, fifth);
	default:		return -alpha_to_intel_errno(EINVAL);
    }
}


int do_semop(unsigned int version, int semid, int nsops, int third, struct sembuf * sops, int fifth)
{
    int		retval;

    /* struct sembuf is the same on alpha and x86, so no translation req'd here */
    retval = semop(semid, sops, nsops);
    if(retval == -1) {
	return(-alpha_to_intel_errno(errno));
    }
    else {
	return(retval);
    }
}

int do_semget(unsigned int version, int key, int nsems, int semflg, void * ptr, int fifth)
{
    int		retval;

    /* Three integer arguments, definitions are the same on x86 and Alpha, no translation 
     * required here.
     */
    retval = semget(key, nsems, semflg);
    if(retval == -1) {
	return(-alpha_to_intel_errno(errno));
    }
    else {
	return(retval);
    }
}


int do_semctl(unsigned int version, int semid, int semun, int cmd, union x86_semun * xsmp, int fifth)
{
    int retval;
    union x86_semun	xsm;
    union semun		sm;

    /* The fourth arg to semctl is a pointer to a union that's either a
     * 32-bit pointer or an int.
     */
    xsm = *xsmp;
    sm.buf = (struct semid_ds *)xsm.buf;
    
    retval = semctl(semid, semun, cmd, sm);
    if(retval == -1) {
	return(-alpha_to_intel_errno(errno));
    }
    else {
        xsm.buf = (unsigned int)((unsigned long)sm.buf & 0xffffffff);
	return(retval);
    }
}

int do_msgsnd(unsigned int version, int msqid, int msgsz, int msgflg, struct x86_msgbuf *xmb, int fifth)
{
	/* mtype is 8 bytes on alpha, 4 bytes in on x86... */
	struct alpha_msgbuf amb;
	int retval;

	amb.mtype = xmb->mtype;
	memcpy(amb.mtext, xmb->mtext, msgsz);
	retval = msgsnd (msqid, &amb, msgsz, msgflg);
	if(retval == -1) {
	    return(-alpha_to_intel_errno(errno));
	}
	else {
	    return(retval);
	}
}

int do_msgrcv(unsigned int version, int msqid, int msgsz, int msgflg, void * ptr, int msgtyp)
{
    struct alpha_msgbuf amb;
    int retval;

    if(version == 0) {
	struct x86_ipc_kludge *tmp = (struct x86_ipc_kludge *)ptr;
	struct x86_msgbuf * x86_msgbp = (struct x86_msgbuf *)((unsigned long)tmp->msgp);
	if(!tmp)
		return -EINVAL;
	retval = msgrcv(msqid, &amb, msgsz, tmp->msgtyp, msgflg);
	if(retval != -1) {
		x86_msgbp->mtype = amb.mtype;
		memcpy(x86_msgbp->mtext, amb.mtext, msgsz);
		return(retval);
	}
	else {
		return(-alpha_to_intel_errno(errno));
	}
    }
    else {
	struct x86_msgbuf * x86_msgbp = (struct x86_msgbuf *)(ptr);
	retval = msgrcv(msqid, &amb, msgsz, msgtyp, msgflg);
	if(retval != -1) {
		x86_msgbp->mtype = amb.mtype;
		memcpy(x86_msgbp->mtext, amb.mtext, msgsz);
		return(retval);
	}
	else {
		return(-alpha_to_intel_errno(errno));
	}
    }
}

int do_msgget(unsigned int version, key_t key, int msgflg, int third, void * ptr, int fifth)
{
    int retval;

    retval = msgget(key, msgflg);
    if(retval == -1) {
	return(-alpha_to_intel_errno(errno));
    }
    else {
	return(retval);
    }
}

int do_msgctl(unsigned int version, int msqid, int cmd, int third, struct x86_msqid_ds * buf, int fifth)
{
    int retval;
    struct msqid_ds	abuf;

    switch(cmd) {
	case X86_IPC_RMID:
	    /* This one is straightforward; buf arg is not used */
	    retval = msgctl(msqid, cmd, (struct msqid_ds *)0);
	    break;

	case X86_IPC_SET:
	    /* Only certain fields can be set; rest are ignored. */
	    retval = msgctl(msqid, IPC_STAT, &abuf);
	    if(retval != -1) {
		abuf.msg_perm.uid = buf->msg_perm.uid;
		abuf.msg_perm.gid = buf->msg_perm.gid;
		abuf.msg_perm.mode &= 0xffe0;
		abuf.msg_perm.mode |= (buf->msg_perm.mode & 0x1f);
		abuf.msg_qbytes = buf->msg_qbytes;

		retval = msgctl(msqid, IPC_SET, &abuf);
	    }
	    break;

	case X86_IPC_STAT:
            /* IPC_STAT *looks* tricky because it's got a coupla
             * pointers in it.  However, it turns out that these
             * pointers are always filled in with *kernel*
             * addresses that the user can never get to -- hence,
             * no working program should ever try to traverse
             * this queue.  Which means we can fill them in
             * with anything we want 8-)
             */

            /* First, get the data */
            retval = msgctl(msqid, IPC_STAT, &abuf);

            /* If we returned OK, then copy it over */
            if(retval != -1) {
                buf->msg_perm.uid = abuf.msg_perm.uid;
                buf->msg_perm.gid = abuf.msg_perm.gid;
                buf->msg_perm.cuid = abuf.msg_perm.cuid;
                buf->msg_perm.cgid = abuf.msg_perm.cgid;
                buf->msg_perm.mode = abuf.msg_perm.mode;
#ifdef RH_40
                buf->msg_perm.seq = abuf.msg_perm.seq;
                buf->msg_perm.key = abuf.msg_perm.key;
#else
                buf->msg_perm.seq = abuf.msg_perm.__seq;
                buf->msg_perm.key = abuf.msg_perm.__key;
#endif
                buf->msg_first = 0xbad00bad;
                buf->msg_last = 0xbad00bad;
		buf->wwait = 0xbad00bad;
		buf->rwait = 0xbad00bad;
                buf->msg_cbytes = abuf.msg_cbytes;
                buf->msg_qnum = abuf.msg_qnum;
                buf->msg_qbytes = abuf.msg_qbytes;
                buf->msg_lspid = abuf.msg_lspid;
                buf->msg_lrpid = abuf.msg_lrpid;
                buf->msg_stime = abuf.msg_stime;
                buf->msg_rtime = abuf.msg_rtime;
                buf->msg_ctime = abuf.msg_ctime;
            }
            break;
	
	default:
	    return(-alpha_to_intel_errno(EINVAL));
	    break;
    }
    if(retval == -1) {
	return(-alpha_to_intel_errno(errno));
    }
    else {
	return(retval);
    }
}

int do_shmat(unsigned int version, int shmid, int shmflg, unsigned int * retaddr, char * shmaddr, int fifth)
{
    char * retval;

    retval = shmat(shmid, shmaddr, shmflg);
    if(retval != ((char *)-1)) {
	*retaddr = (unsigned int)((unsigned long)retval & 0xffffffff);
	return retval;
    }
    else {
	return(-alpha_to_intel_errno(errno));
    }
}

int do_shmdt(unsigned int version, int first, int second, int third, char *shmaddr, int fifth)
{
    int retval;

    retval = shmdt(shmaddr);
    if(retval == -1) {
	return(-alpha_to_intel_errno(errno));
    }
    else {
	return(retval);
    }
}

int do_shmget(unsigned int version, int key, int size, int shmflg, void * ptr, int fifth)
{
    int retval;

    retval = shmget(key, size, shmflg);
    if(retval == -1) {
	return(-alpha_to_intel_errno(errno));
    }
    else {
	return(retval);
    }
}

int do_shmctl(unsigned int version, int shmid, int cmd, int third, struct x86_shmid_ds * xbuf, int fifth)
{
    struct shmid_ds	abuf;
    int retval;

    switch(cmd) {
	case X86_IPC_RMID:
	    retval = shmctl(shmid, cmd, (void *)0);
	    break;
	case X86_IPC_STAT:
	    retval = shmctl(shmid, IPC_STAT, &abuf);
	    if(retval != -1) {
                xbuf->shm_perm.uid = abuf.shm_perm.uid;
                xbuf->shm_perm.gid = abuf.shm_perm.gid;
                xbuf->shm_perm.cuid = abuf.shm_perm.cuid;
                xbuf->shm_perm.cgid = abuf.shm_perm.cgid;
                xbuf->shm_perm.mode = abuf.shm_perm.mode;
#ifdef RH_40
                xbuf->shm_perm.seq = abuf.shm_perm.seq;
                xbuf->shm_perm.key = abuf.shm_perm.key;
#else
                xbuf->shm_perm.seq = abuf.shm_perm.__seq;
                xbuf->shm_perm.key = abuf.shm_perm.__key;
#endif
		xbuf->shm_atime = abuf.shm_atime;
		xbuf->shm_dtime = abuf.shm_dtime;
		xbuf->shm_ctime = abuf.shm_ctime;
		xbuf->shm_cpid = abuf.shm_cpid;
		xbuf->shm_lpid = abuf.shm_lpid;
		xbuf->shm_nattch = abuf.shm_nattch;
#ifdef RH_40
		xbuf->shm_npages = abuf.shm_npages;
#else
		xbuf->shm_npages = abuf.__shm_npages;
#endif
		xbuf->shm_pages = 0xbad00bad;
		xbuf->attaches = 0xbad00bad;
	    }
	    break;
	case X86_IPC_SET:
	    retval = shmctl(shmid, IPC_STAT, &abuf);
	    if(retval != -1) {
		abuf.shm_perm.uid = xbuf->shm_perm.uid;
		abuf.shm_perm.gid = xbuf->shm_perm.gid;
		abuf.shm_perm.cuid = xbuf->shm_perm.cuid;
		abuf.shm_perm.cgid = xbuf->shm_perm.cgid;
		abuf.shm_perm.mode = xbuf->shm_perm.mode;
#ifdef RH_40
		abuf.shm_perm.seq = xbuf->shm_perm.seq;
		abuf.shm_perm.key = xbuf->shm_perm.key;
#else
		abuf.shm_perm.__seq = xbuf->shm_perm.seq;
		abuf.shm_perm.__key = xbuf->shm_perm.key;
#endif
		abuf.shm_ctime = xbuf->shm_ctime;
		retval = shmctl(shmid, IPC_SET, &abuf);
	    }
	    break;
	default:
	    return(-alpha_to_intel_errno(EINVAL));
    }
    if(retval == -1) {
	return(-alpha_to_intel_errno(errno));
    }
    else {
	return(retval);
    }
}
