#include <netdb.h>
#include <stdlib.h>

/* Structure definitions for complicated libc thunking... */

struct x86_hostent {
    unsigned int	h_name;
    unsigned int	h_aliases;
    int			h_addrtype;
    int			h_length;
    unsigned int	h_addr_list;
};

static struct x86_hostent	xent;
static int h_alias_count = -1;
static unsigned int * h_alias_array = 0;
static int h_addr_count = -1;
static unsigned int * h_addr_array = 0;

struct x86_hostent * alpha_to_x86_hostent(struct hostent *aent) 
{
    int	i, xh_alias_count, xh_addr_count;

    if(!aent) {
	return((void *)0);
    }

    /* Count number of entries in alias and addr lists */
    for(xh_alias_count=0; aent->h_aliases[xh_alias_count]; xh_alias_count++) ;
    for(xh_addr_count=0; aent->h_addr_list[xh_addr_count]; xh_addr_count++) ;

    /* Make sure the lists are big enough to hold what we need... */
    if(xh_alias_count > h_alias_count) {
	if(h_alias_array) {
	    free(h_alias_array);
	}
	h_alias_array = malloc((xh_alias_count+1) * sizeof(unsigned int));
	h_alias_count = xh_alias_count;
    }

    if(xh_addr_count > h_addr_count) {
	if(h_addr_array) {
	    free(h_addr_array);
	}
	h_addr_array = malloc((xh_addr_count+1) * sizeof(unsigned int));
	h_addr_count = xh_addr_count;
    }

    /* Copy the data over */
    xent.h_name = (unsigned int)((unsigned long)aent->h_name & 0xffffffff);
    for(i = 0; i < xh_alias_count; i++) {
	h_alias_array[i] = 
	    (unsigned int)((unsigned long)aent->h_aliases[i] & 0xffffffff);
    }
    h_alias_array[i] = 0;
    xent.h_aliases = (unsigned int)((unsigned long)h_alias_array & 0xffffffff);
    xent.h_addrtype = aent->h_addrtype;
    xent.h_length = aent->h_length;
    for(i = 0; i < xh_addr_count; i++) {
	h_addr_array[i] = 
	    (unsigned int)((unsigned long)aent->h_addr_list[i] & 0xffffffff);
    }
    h_addr_array[i] = 0;
    xent.h_addr_list = (unsigned int)((unsigned long)h_addr_array & 0xffffffff);
    return(&xent);
}

struct x86_hostent *
do_gethostbyname(char * name)
{
    return(alpha_to_x86_hostent(gethostbyname(name)));
}

