#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

// #define PRINTF_DEBUG
#define STACKSIZE 1024
#define MAXSPECSIZE 10
#define FPDIFF 48

extern FILE * em86_sync_in(void *);
extern void em86_sync_out(void *, FILE *);

enum { PRINTF, FPRINTF, SPRINTF, SNPRINTF };

int do_printf (char *fmt, unsigned long argstack)
{
  return do_printf_main (NULL, NULL, 0, fmt, argstack, PRINTF);
}

int do_fprintf (void *fp, char *fmt, unsigned long argstack)
{
  return do_printf_main (fp, NULL, 0, fmt, argstack, FPRINTF);
}

int do_sprintf (char *str, char *fmt, unsigned long argstack)
{
  return do_printf_main (NULL, str, 0, fmt, argstack, SPRINTF);
}

int do_snprintf (char *str, size_t size, char *fmt, unsigned long argstack)
{
  return do_printf_main (NULL, str, size, fmt, argstack, SNPRINTF);
}

int do_vprintf (char *fmt, unsigned long argstack)
{
  return do_printf_main (NULL, NULL, 0, fmt, argstack, PRINTF);
}

int do_vfprintf (void *fp, char *fmt, unsigned long argstack)
{
  return do_printf_main (fp, NULL, 0, fmt, argstack, FPRINTF);
}

int do_vsprintf (char *str, char *fmt, unsigned long argstack)
{
  return do_printf_main (NULL, str, 0, fmt, argstack, SPRINTF);
}

int do_vsnprintf (char *str, size_t size, char *fmt, unsigned long argstack)
{
  return do_printf_main (NULL, str, size, fmt, argstack, SNPRINTF);
}

int find_width (char *start, char *end)
{
  int num = 0;
  char *p;

  for (p = start; p < end; p++)
    if (*p == '*')
      num++;

  return num;
}

char *find_mod (char *start, char *end, char *srch)
{
  static char buf [MAXSPECSIZE];
  char *c;
  
  memcpy (buf, start, end - start);
  buf [end - start] = '\0';

  return strpbrk (buf, srch);
}

int do_printf_main (void *fp, char *str, size_t size, 
		    char *fmt, unsigned long argstack, int type)
{
  va_list ap;
  char *ptr, *spec, *mod, *sp;
  char stack [STACKSIZE];
  int x86count = 0, alphacount = 0, num;
  long tmp;
  double tmpfl;
  FILE * afp;
  int retval;

  if(fp) {
     afp = em86_sync_in(fp);
  }
  sp = (char *) argstack;

#ifdef PRINTF_DEBUG
  printf ("ALPHA: fmt = %p\n", fmt);
#endif

  /* For each specifier, copy the appropriate number of bytes */
  for (ptr = fmt; *ptr; ptr++) {
    if (*ptr == '%') {
      if (*(ptr + 1) == '%') {
	ptr += 2;
	continue;
      }

      spec = strpbrk (ptr, "diouxXeEfgcspn");

      /* This code finds those pesky * values for field width
       * which have to be taken as int args and converted
       */
      num = find_width (ptr, spec);

      while (num--) {
#ifdef PRINTF_DEBUG
	printf ("ALPHA: Copying 32-bit int\n");
#endif
	tmp = (unsigned long) *((unsigned int *)(sp + x86count))
	  & 0xffffffff; 
	memcpy (&(stack [alphacount + FPDIFF]), &tmp, 8);
	x86count += 4;
	alphacount += 8;
      }

      switch (*spec)
	{
	case 'd':
	case 'i':
	case 'o':
	case 'u':
	case 'x':
	case 'X':
	  /* Modifiers are h (short), l (long), {q, L} (long long),
	   * Z (size_t)
	   */
	  
	  if ((mod = find_mod (ptr, spec, "hlqLZ")) == NULL) {
#ifdef PRINTF_DEBUG
	    printf ("ALPHA: Copying 32-bit int\n");
#endif
	    tmp = (unsigned long) *((unsigned int *)(sp + x86count))
	      & 0xffffffff; 
	    memcpy (&(stack [alphacount + FPDIFF]), &tmp, 8);
	    x86count += 4;
	    alphacount += 8;
	  } else {
	    switch (*mod)
	      {
	      case 'h':
#ifdef PRINTF_DEBUG
		printf ("ALPHA: Copying 16-bit short\n");
#endif
		tmp = (unsigned long) *((unsigned int *)(sp + x86count))
		  & 0xffffffff; 
		memcpy (&(stack [alphacount + FPDIFF]), &tmp, 8);
		x86count += 4;
		alphacount += 8;
		break;
	      case 'q':
	      case 'L':
#ifdef PRINTF_DEBUG
		printf ("ALPHA: Copying 64-bit long long\n");
#endif
		memcpy (&(stack [alphacount + FPDIFF]), sp + x86count, 8);
		x86count += 8;
		alphacount += 8;
		break;
	      case 'l':
	      case 'Z':
#ifdef PRINTF_DEBUG
		printf ("ALPHA: Converting 32-bit long/size_t to 64-bit\n");
#endif
		if (*spec != 'd' && *spec != 'i')
		  tmp = (unsigned long) *((unsigned int *)(sp + x86count))
		    & 0xffffffff;
		else
		  tmp = (long) *((int *)(sp + x86count));
		memcpy (&(stack [alphacount + FPDIFF]), &tmp, 8);
		x86count += 4;
		alphacount += 8;
		break;
	      }
	  }
	  break;
	case 'e':
	case 'E':
	case 'f':
	case 'g':
	case 'G':
	  /* Modifiers are {q, L} (long double) */
	  if ((mod = find_mod (ptr, spec, "qL")) == NULL) {
#ifdef PRINTF_DEBUG
	    printf ("ALPHA: Copying 64-bit double\n");
#endif
	    /*
	      if ((((unsigned long)(sp + x86count)) & 7) != 0) {
	      memcpy (bufhi, sp + x86count, 4);
	      tmpfl = 
	      (double)(((unsigned long)*((int *)(sp + x86count)) << 32) | 
	      *((int *)(sp + x86count + 4)));
	      else
	      tmpfl = *((double *)(sp + x86count)); 
	      */

	    if (alphacount <= (6 * 8)) 
	      memcpy (&(stack [alphacount]), sp + x86count, 8);
	    else 
	      memcpy (&(stack [alphacount + FPDIFF]), sp + x86count, 8);
	    x86count += 8;
	    alphacount += 8;
	  } else {
	    fprintf (stderr, "libc-assist: error - tried to thunk x86 long double to alpha long double\n");
	    return -1;
	  }
	  break;
	case 'c':
	case 's':
	case 'p':
	case 'n':
#ifdef PRINTF_DEBUG
	  printf ("ALPHA: Converting 32-bit value to 64-bit\n");
#endif
	  tmp = (unsigned long) *((unsigned int *)(sp + x86count))
	    & 0xffffffff; 
	  memcpy (&(stack [alphacount + FPDIFF]), &tmp, 8);
	  x86count += 4;
	  alphacount += 8;
	  break;
	}
    }
  }

#ifdef PRINTF_DEBUG
  printf ("ALPHA: stack = %p\n", stack);
#endif

  ap.__base = stack + FPDIFF;
  ap.__offset = 0;

  switch (type)
    {
    case PRINTF:    retval = vprintf (fmt, ap); break;
    case FPRINTF:   retval = vfprintf (afp, fmt, ap); break;
    case SPRINTF:   retval = vsprintf (str, fmt, ap); break;
    case SNPRINTF:  retval = vsnprintf (str, size, fmt, ap); break;
    }
    if(fp) {
	em86_sync_out(fp, afp);
    }
    return retval;
}
	  

	  
	  
