/* This routine builds a "reverse thunk" for calling Intel code from within
 * an Alpha routine.
 */
#include <stdlib.h>
#include <dlfcn.h>
#include <link.h>

static void * intel_emulator_addr = 0;

extern unsigned int em86_call_emulated_routine();

void * build_reverse_thunk(void * iaddr)
{
    unsigned int *	thkaddr;

    if(!intel_emulator_addr) {
	/* We need to fetch the Intel emulator address... */
	void *	main_handle;
	main_handle = dlopen("", 0);	/* Main program */
	if(!main_handle) {
	    printf("dlopen error: %s\n", dlerror());
	}
	intel_emulator_addr = dlsym(main_handle, "em86_call_emulated_routine");
	if(!intel_emulator_addr) {
	    printf("Cannot find address for em86_call_emulated_routine: %s\n",
			dlerror());
	    abort();
	}
#ifdef VERBOSE
	else {
	    printf("em86_call_emulated_routine is at 0x%lx\n", intel_emulator_addr);
	}
#endif
    }

    /* Allocate memory for the thunk, then build it */
    thkaddr = malloc(10 * sizeof(unsigned int));

    thkaddr[0] = 0x23defff8;		/* lda	sp,-8(sp) */
    thkaddr[1] = 0xb75e0000;		/* stq	sp,0(sp) */
    thkaddr[2] = 0x26bf0000 | 		/* ldah a5,iaddr_HI(zero) */
		(((unsigned long)iaddr >> 16) & 0xffff);
    if((unsigned long)iaddr & 0x8000) thkaddr[2]++;

    thkaddr[3] = 0x22b50000 |		/* lda 	a5,iaddr_LO(a5) */
		((unsigned long)iaddr & 0xffff);
    thkaddr[4] = 0x277f0000 |		/* ldah	pv,emuaddr_HI(zero) */
		(((unsigned long)intel_emulator_addr >> 16) & 0xffff);
    if((unsigned long)intel_emulator_addr & 0x8000) thkaddr[4]++;

    thkaddr[5] = 0x237b0000 |		/* lda	pv,emuaddr_LO(pv) */
		((unsigned long)intel_emulator_addr & 0xffff);
    thkaddr[6] = 0x6b5b4000;		/* jsr	ra,(pv) */
    thkaddr[7] = 0xa75e0000;		/* ldq	ra,0(sp) */
    thkaddr[8] = 0x23de0008;		/* lda	sp,8(sp) */
    thkaddr[9] = 0x6bfa8000;		/* ret	zero,(ra) */

#ifdef VERBOSE
    printf("build_reverse_thunk: returning thunk @0x%x\n", thkaddr);
#endif

    return(thkaddr);
}

void free_reverse_thunk(void *thkaddr)
{
    free(thkaddr);
}

/* Utility function used to determine if we need a reverse thunk or not */
int is_alpha_code(void * addr)
{
	int i;
	struct link_map *	lm;
	Elf64_Phdr *	ph;

	if(_dl_loaded == (void *)0) {
	    return 0;
	}

	for(lm = _dl_loaded; lm; lm = lm->l_next) {
		if(lm->l_phdr) {
		    ph = lm->l_phdr;
		    for(i = 0; i < lm->l_phnum; i++) {
			void *base = lm->l_addr + ph->p_vaddr;
			void *top = base + ph->p_memsz;
			if(addr >= base && addr < top) {
			    return 1;
			}
			ph++;
		    }
		}
	}

	return 0;

}

