#include <stdio.h>

/* These functions handle a sticky problem: how to deal with the
 * FILE* arguments to stdio functions.  There are two problems
 * here.  First is that there are three pre-declared structs
 * in the x86 code (stdin, stdout, stderr).  Second is that the
 * FILE* is not a completely opaque pointer; there are some macros
 * that generate inline x86 code to access this struct directly.
 */

struct x86_IO_lock_t {
	unsigned int	ptr;		/* really void * */
	short int	field1;
	short int	field2;
};

typedef struct x86_IO_FILE {
	int		_flags;
	unsigned int	_IO_read_ptr;	/* really (char *) */

	unsigned int	_IO_read_end;	/* really (char *) */
	unsigned int	_IO_read_base;	/* really (char *) */

	unsigned int	_IO_write_base;	/* really (char *) */
	unsigned int	_IO_write_ptr;	/* really (char *) */

	unsigned int	_IO_write_end;	/* really (char *) */
	unsigned int	_IO_buf_base;	/* really (char *) */

	unsigned int	_IO_buf_end;	/* really (char *) */
	unsigned int	_IO_save_base;	/* really (char *) */

	unsigned int	_IO_backup_base;/* really (char *) */
	unsigned int	_IO_save_end;	/* really (char *) */

	/* This is a bit of a crock... we're going to usurp some of
	 * these fields for our own use.  These are the fields that
	 * are *supposed* to be here at this point...
	 * unsigned int	_markers;
	 * unsigned int	_chaint;
	 * int		_fileno;
	 * int		_blksize;
	 */
	FILE *		_alpha_IO_FILE;
	unsigned long	_malloc_magic;	/* This is to show that the struct
					 * was malloc'ed and therefore can
					 * be freed on close.
					 */

	/* end of crock */	
	int		_offset;	/* really _IO_off_t */
	unsigned short	_cur_column;
	char		_unused;
	char		_shortbuf[1];
	struct x86_IO_lock_t	_IO_lock;
} x86_FILE;

#define MALLOC_MAGIC	0x656d38366861636b


void em86_init_default_streams(x86_FILE *in, x86_FILE *out, x86_FILE *err)
{
    in->_alpha_IO_FILE = stdin;
    in->_malloc_magic = 0;
    out->_alpha_IO_FILE = stdout;
    out->_malloc_magic = 0;
    err->_alpha_IO_FILE = stderr;
    err->_malloc_magic = 0;
}

/* This function is called when an alpha thunk is about to use an x86
 * io struct.  Sync up the important fields, then return the pointer
 * to the alpha io struct.
 */
FILE * em86_sync_in(x86_FILE *xp)
{
    FILE *	ap;

    ap = (FILE *)((unsigned long)xp->_alpha_IO_FILE);
    ap->_IO_read_ptr = (char *)((unsigned long)xp->_IO_read_ptr);
    ap->_IO_read_end = (char *)((unsigned long)xp->_IO_read_end);
    ap->_IO_read_base = (char *)((unsigned long)xp->_IO_read_base);
    ap->_IO_write_ptr = (char *)((unsigned long)xp->_IO_write_ptr);
    ap->_IO_write_end = (char *)((unsigned long)xp->_IO_write_end);
    ap->_IO_write_base = (char *)((unsigned long)xp->_IO_write_base);
    ap->_IO_buf_end = (char *)((unsigned long)xp->_IO_buf_end);
    ap->_IO_buf_base = (char *)((unsigned long)xp->_IO_buf_base);
    ap->_IO_save_end = (char *)((unsigned long)xp->_IO_save_end);
    ap->_IO_save_base = (char *)((unsigned long)xp->_IO_save_base);
    ap->_IO_backup_base = (char *)((unsigned long)xp->_IO_backup_base);
    ap->_flags = xp->_flags;
    return(ap);
}

/* This function is called when a thunked function returns.  It updates
 * the critical fields in the x86 struct with the equivalent alpha fields.
 */
void em86_sync_out(x86_FILE *xp, FILE * ap)
{
    xp->_IO_read_ptr = (unsigned int)((unsigned long)ap->_IO_read_ptr & 0xffffffff);
    xp->_IO_read_end = (unsigned int)((unsigned long)ap->_IO_read_end & 0xffffffff);
    xp->_IO_read_base = (unsigned int)((unsigned long)ap->_IO_read_base & 0xffffffff);
    xp->_IO_write_ptr = (unsigned int)((unsigned long)ap->_IO_write_ptr & 0xffffffff);
    xp->_IO_write_end = (unsigned int)((unsigned long)ap->_IO_write_end & 0xffffffff);
    xp->_IO_write_base = (unsigned int)((unsigned long)ap->_IO_write_base & 0xffffffff);
    xp->_IO_buf_end = (unsigned int)((unsigned long)ap->_IO_buf_end & 0xffffffff);
    xp->_IO_buf_base = (unsigned int)((unsigned long)ap->_IO_buf_base & 0xffffffff);
    xp->_IO_save_end = (unsigned int)((unsigned long)ap->_IO_save_end & 0xffffffff);
    xp->_IO_save_base = (unsigned int)((unsigned long)ap->_IO_save_base & 0xffffffff);
    xp->_IO_backup_base = (unsigned int)((unsigned long)ap->_IO_backup_base & 0xffffffff);
    xp->_flags = ap->_flags;
}



/* These are all the functions that need the special treatment... */

x86_FILE *do_freopen( char *path, char *mode, x86_FILE *stream) 
{
    FILE *	alpha_stream;
    x86_FILE *	x86_stream;

    alpha_stream = freopen(path, mode, em86_sync_in(stream));

    if(alpha_stream) {
	x86_stream = (x86_FILE *)malloc(sizeof(x86_FILE));
        x86_stream->_alpha_IO_FILE = alpha_stream;
	x86_stream->_malloc_magic = MALLOC_MAGIC;
	em86_sync_out(x86_stream, alpha_stream);
	return(x86_stream);
    }
    else {
	return 0;
    }
}

x86_FILE *do_fopen( char *path, char *mode)
{
    FILE *	alpha_stream;
    x86_FILE *	x86_stream;

    alpha_stream = fopen(path, mode);

    if(alpha_stream) {
	x86_stream = (x86_FILE *)malloc(sizeof(x86_FILE));
        x86_stream->_alpha_IO_FILE = alpha_stream;
	x86_stream->_malloc_magic = MALLOC_MAGIC;
	em86_sync_out(x86_stream, alpha_stream);
	return(x86_stream);
    }
    else {
	return 0;
    }
}

x86_FILE *do_tmpfile( char *path, char *mode)
{
    FILE *	alpha_stream;
    x86_FILE *	x86_stream;

    alpha_stream = tmpfile();

    if(alpha_stream) {
	x86_stream = (x86_FILE *)malloc(sizeof(x86_FILE));
        x86_stream->_alpha_IO_FILE = alpha_stream;
	x86_stream->_malloc_magic = MALLOC_MAGIC;
	em86_sync_out(x86_stream, alpha_stream);
	return(x86_stream);
    }
    else {
	return 0;
    }
}

x86_FILE *do_fdopen( int fd, char *mode)
{
    FILE *	alpha_stream;
    x86_FILE *	x86_stream;

    alpha_stream = fdopen(fd, mode);

    if(alpha_stream) {
	x86_stream = (x86_FILE *)malloc(sizeof(x86_FILE));
        x86_stream->_alpha_IO_FILE = alpha_stream;
	x86_stream->_malloc_magic = MALLOC_MAGIC;
	em86_sync_out(x86_stream, alpha_stream);
	return(x86_stream);
    }
    else {
	return 0;
    }
}

x86_FILE *do_popen( char *command, char *type)
{
    FILE *	alpha_stream;
    x86_FILE *	x86_stream;

    alpha_stream = popen(command, type);

    if(alpha_stream) {
	x86_stream = (x86_FILE *)malloc(sizeof(x86_FILE));
        x86_stream->_alpha_IO_FILE = alpha_stream;
	x86_stream->_malloc_magic = MALLOC_MAGIC;
	em86_sync_out(x86_stream, alpha_stream);
	return(x86_stream);
    }
    else {
	return 0;
    }
}

int do_fclose(x86_FILE *stream) 
{
    FILE * alpha_stream;
    int	   retval;

    if(stream) {
	alpha_stream = em86_sync_in(stream);
	retval = fclose(alpha_stream);
	if(stream->_malloc_magic == MALLOC_MAGIC) {
	    free(stream);
	}
	return(retval);
    }
    else {
	return(-1);
    }
}

int do_fflush(x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = fflush(alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_fputc(int c, x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = fputc(c, alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_fputs(const char *s, x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = fputs(s, alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_putc(int c, x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = putc(c, alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_ungetc(int c, x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = ungetc(c, alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_fgetc(x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = fgetc(alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

char *do_fgets(char *s, int size, x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    char *	retval;

    retval = fgets(s, size, alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_getc(x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = getc(alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_fseek( x86_FILE *stream, long offset, int whence)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = fseek(alpha_stream, offset, whence);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

long do_ftell( x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = ftell(alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

void do_rewind( x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);

    rewind(alpha_stream);
    em86_sync_out(stream, alpha_stream);
}

int do_fgetpos( x86_FILE *stream, fpos_t *pos)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = fgetpos(alpha_stream, pos);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_fsetpos( x86_FILE *stream, fpos_t *pos)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = fsetpos(alpha_stream, pos);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

size_t  do_fread(  void *ptr, size_t size, size_t nmemb, x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = fread(ptr, size, nmemb, alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

size_t do_fwrite( void *ptr, size_t size, size_t nmemb,  x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = fwrite(ptr, size, nmemb, alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

void do_clearerr( x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);

    clearerr(alpha_stream);
    em86_sync_out(stream, alpha_stream);
}

int do_feof( x86_FILE *stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = feof(alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_ferror( x86_FILE *stream){
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = ferror(alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_fileno( x86_FILE *stream){
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = fileno(alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

void  do_setbuf( x86_FILE *stream, char *buf){
    FILE *	alpha_stream = em86_sync_in(stream);

    setbuf(alpha_stream, buf);
    em86_sync_out(stream, alpha_stream);
}

void  do_setbuffer( x86_FILE *stream, char *buf, size_t size){
    FILE *	alpha_stream = em86_sync_in(stream);

    setbuffer(alpha_stream, buf, size);
    em86_sync_out(stream, alpha_stream);
}

void do_setlinebuf( x86_FILE *stream){
    FILE *	alpha_stream = em86_sync_in(stream);

    setlinebuf(alpha_stream);
    em86_sync_out(stream, alpha_stream);
}

int do_setvbuf( x86_FILE *stream, char *buf, int mode , size_t size){
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = setvbuf(alpha_stream, buf, mode, size);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_getw(x86_FILE *stream){
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = getw(alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_putw(int w, x86_FILE *stream){
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = putw(w, alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_pclose(x86_FILE *stream){
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = pclose(alpha_stream);
    if(stream->_malloc_magic == MALLOC_MAGIC) {
	free(stream);
    }
    return(retval);
}

int do_getdelim(char ** d, size_t * s, int i, x86_FILE* stream){
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = getdelim(d, s, i, alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do_getline(char ** d, size_t * s, x86_FILE * stream){
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = getline(d, s, alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do__underflow(x86_FILE * stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = __underflow(alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}

int do__uflow(x86_FILE * stream)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = __uflow(alpha_stream);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}


int do__overflow(x86_FILE * stream, int n)
{
    FILE *	alpha_stream = em86_sync_in(stream);
    int		retval;

    retval = __overflow(alpha_stream, n);
    em86_sync_out(stream, alpha_stream);
    return(retval);
}


